/*
For general Scribus (>=1.3.2) copyright and licensing information please refer
to the COPYING file provided with the program. Following this notice may exist
a copyright and/or license notice that predates the release of Scribus 1.3.2
for which a new license (GPL+exception) is in place.
*/

#include "scprintengine_cairo.h"
#include "scribusstructs.h"
#include "scribusdoc.h"
#include "scribuscore.h"
#include "prefsmanager.h"
#include "pslib.h"
#include "util_file.h"
#include "util_ghostscript.h"

#include <cmath>
#include "scpainterex_cairo.h"
#include "sccolorengine.h"
#include "util.h"

#include "scpageoutput.h"
#include "prefsfile.h"
#include "prefscontext.h"
#include "prefsmanager.h"

#include <cairo/cairo-pdf.h>
#include <cairo/cairo-svg.h>
#include <cairo/cairo-ps.h>


static bool printPage ( ScribusDoc* doc, Page* page, PrintOptions& options, cairo_t* context )
{
	int physicalWidth;
	int physicalHeight;
	int physicalWidthP;
	int physicalHeightP;
	bool success = true;
	QString inputProfile;
	QString printerProfile;

	double scalex = 1, scaley = 1;

	// Get page position
	int clipx = static_cast<int>(page->xOffset());
	int clipy = static_cast<int>(page->yOffset());
	int clipw = qRound(page->width());
	int cliph = qRound(page->height());

	// Get paper dimensions ( in pixels and points)
	physicalWidth   = doc->pageWidth;
	physicalHeight  = doc->pageHeight;
	physicalWidthP  = physicalWidth;
	physicalHeightP = physicalHeight;

	// Calculate scaling factors and offsets
	scalex = options.mirrorH ? -1.0 : 1.0;
	scaley = options.mirrorV ? -1.0 : 1.0; 
	double dx = ( physicalWidthP - clipw ) / 2.0 - clipx * scalex;
	double dy = ( physicalHeightP - cliph ) / 2.0 - clipy * scaley;
	if ( options.mirrorH ) dx += clipw;
	if ( options.mirrorV ) dy += cliph;
	 
	// Create the GDI painter
	MarksOptions marksOptions(options);
	ScPageOutput pageOutput(doc, true, 300, false);
	pageOutput.setMarksOptions(marksOptions);
	
	QRect drawRect( 0, 0, physicalWidth, physicalHeight);
	ScPainterEx_Cairo painter(context, drawRect, doc, !options.useColor );
	painter.clear();
	
	QMatrix matrix( scalex, 0.0, 0.0, scaley, dx, dy );
	painter.setWorldMatrix( matrix );

	pageOutput.drawPage(page, &painter);

	cairo_show_page(context);

	return success;
}

bool ScPrintEngine_Cairo::print(ScribusDoc& doc, PrintOptions& options)
{
	Page* docPage;
	bool success = true;

	PrefsManager *prefsManager = PrefsManager::instance();
	QString filename(options.filename);
	if (!options.toFile)
		filename = prefsManager->preferencesLocation()+"/tmp.pdf";
	// Setup document infos structure

	cairo_surface_t* prnSurface;
	if(filename.toLower().endsWith(".pdf"))
		prnSurface = cairo_pdf_surface_create (filename.toLocal8Bit(), doc.pageWidth, doc.pageHeight);
	else if(filename.toLower().endsWith(".svg"))
		prnSurface = cairo_svg_surface_create (filename.toLocal8Bit(), doc.pageWidth, doc.pageHeight);
	else
		prnSurface = cairo_ps_surface_create (filename.toLocal8Bit(), doc.pageWidth, doc.pageHeight);

	cairo_t* context            = cairo_create (prnSurface);
	if (!context){
		cairo_surface_destroy( prnSurface );
		return false;
	}

	for ( uint index = 0; index < options.pageNumbers.size(); index++ )
	{
		docPage = doc.Pages->at( options.pageNumbers[index] - 1 );
		success = printPage( &doc, docPage, options, context );
		if (!success )
			break;
	}

	cairo_destroy( context );
	cairo_surface_destroy( prnSurface );

	return success;
}

